
# q-expansion of newform 804.2.i.e, downloaded from the LMFDB on 08 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 804
weight = 2
poly_data = [9, 24, 85, -50, 54, -9, 8, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0], 1], [[9771, 1485, 4905, 17278, 180, 2025, 154, 315], 32593], [[13524, 55363, -292202, 97026, -59463, 7346, -8503, 1867], 97779], [[-73713, -262130, 214684, -175338, 35388, -25594, 4202, -3257], 97779], [[13496, -5214, -17222, -33142, -632, -7110, -1265, -1106], 32593], [[-123812, -8910, -29430, -71075, -1080, -12150, -924, -1890], 32593], [[-60795, 1068661, -542330, 442935, -84789, 64655, -10615, 6436], 97779]]

hecke_ring_character_values = [[403, [1, 0, 0, 0, 0, 0, 0, 0]], [269, [1, 0, 0, 0, 0, 0, 0, 0]], [337, [0, 0, 0, 0, 1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0], [1, 0, 0, 0, 0, 0, 0, 0], [-1, 0, -1, 0, 0, 1, -1, 0], [0, 0, 0, -1, 0, 0, 0, 0], [0, 1, 1, 0, -1, 0, -1, 1], [0, -2, 0, 0, 0, 0, 0, 1], [0, -1, 0, 0, 0, 0, 0, 0], [1, 0, 0, -1, 1, 1, 0, 1], [3, 0, 0, 0, 3, 0, 0, 0], [0, 1, 1, -1, 2, 0, 1, -1], [0, -3, -3, 0, 1, 0, -1, 1], [4, -2, 0, -2, 4, 2, 0, -1], [0, -2, -2, 0, -2, 0, 1, -1], [1, 0, 0, 0, 0, 0, 1, 0], [0, 0, 0, 0, -4, 0, -1, 1], [-1, 0, 2, 0, 0, -4, 3, 0], [0, 0, 0, 0, 0, 1, 1, 0], [0, 4, 0, -3, 0, 3, 0, -2], [1, 2, 1, 2, -3, -1, -1, -1], [0, -2, -2, 0, -1, 0, 2, -2], [-4, 3, 0, 0, -4, 0, 0, 2], [0, 2, 2, 0, 4, 0, -1, 1], [-4, 1, 0, 0, -4, 0, 0, -2], [-1, 0, -2, 0, 0, 1, -4, 0], [8, -2, 0, 3, 8, -3, 0, 2], [0, -3, -3, 4, 8, 0, 0, 0], [0, 2, 2, -3, -9, 0, 1, -1], [3, 0, 5, 0, 0, -1, 5, 0], [4, 0, -1, 0, 0, 1, -4, 0], [0, 0, 0, 1, 6, 0, 4, -4], [0, 2, 2, -3, 1, 0, -1, 1], [-3, 0, -1, 0, 0, 5, -1, 0], [-5, 0, -5, 0, 0, -2, 0, 0], [3, 0, -3, 0, 0, 1, 3, 0], [-1, 0, 6, 0, 0, -5, 2, 0], [1, -7, 0, 5, 1, -5, 0, 4], [-5, 3, 0, -2, -5, 2, 0, -1], [0, 1, 1, 1, -4, 0, 1, -1], [0, -7, -7, 2, 4, 0, 0, 0], [-6, 4, 0, 5, -6, -5, 0, -2], [-5, 0, 10, 0, 0, -1, 4, 0], [0, -1, -1, -4, -3, 0, 3, -3], [3, 4, 0, -1, 3, 1, 0, 1], [-1, 0, -4, 0, 0, -3, 6, 0], [0, -4, -4, 1, -1, 0, 3, -3], [6, -3, 0, -1, 6, 1, 0, -1], [1, 5, 0, 0, 1, 0, 0, -3], [-5, 0, -4, 0, 0, 3, -2, 0], [0, -2, -2, 9, 0, 0, -6, 6], [0, 4, 0, 0, 0, 0, 0, -5], [0, -3, -3, 8, 4, 0, 0, 0], [0, 0, 0, 3, 5, 0, -7, 7], [6, 0, 2, 0, 0, -1, 1, 0], [-13, 4, 0, -7, -13, 7, 0, -1], [0, 4, 4, -7, -7, 0, 1, -1], [1, 0, -7, 0, 0, 2, -2, 0], [-6, 0, -5, 0, 0, 0, -3, 0], [4, 0, 9, 0, 0, -6, 1, 0], [1, 0, 5, 0, 0, -6, 4, 0], [1, 6, 0, -2, 1, 2, 0, -2], [6, 0, 4, 0, 0, -7, 7, 0], [8, 0, 0, 0, 0, -5, 5, 0], [13, 2, 0, 1, 13, -1, 0, 5], [7, 0, 9, 0, 0, -7, 7, 0], [-8, 0, -7, 0, 0, 6, -5, 0], [-7, 3, 0, -9, -7, 9, 0, -2], [0, 3, 3, -5, -17, 0, 6, -6], [1, -7, 0, -3, 1, 3, 0, 0], [0, -6, -6, -6, 1, 0, -2, 2], [-2, 0, -8, 0, 0, -2, 2, 0], [1, -5, 0, -3, 1, 3, 0, -4], [20, 0, 0, 0, 0, 5, -3, 0], [0, -2, -2, 0, -4, 0, -9, 9], [0, 7, 7, 6, -7, 0, -3, 3], [-2, 0, 0, -6, -2, 6, 0, 3], [0, 2, 0, -6, 0, 6, 0, -3], [23, -3, 0, -2, 23, 2, 0, 3], [5, 0, -6, 0, 0, 8, -7, 0], [-11, 0, 8, 0, 0, -1, -2, 0], [0, -1, -1, 5, -8, 0, 1, -1], [-2, -3, 0, -1, -2, 1, 0, 9], [-2, 10, 0, -7, -2, 7, 0, -2], [0, -1, -1, -7, 2, 0, -5, 5], [0, -8, -8, 2, 5, 0, -4, 4], [1, 9, 0, -9, 1, 9, 0, -6], [0, 10, 10, 1, -4, 0, 6, -6], [0, 1, 1, -4, 3, 0, -7, 7], [0, 5, 5, 8, -7, 0, -7, 7], [20, 0, 8, 0, 0, 0, -2, 0], [-3, 12, 0, 3, -3, -3, 0, -11], [7, -10, 0, 0, 7, 0, 0, 2], [-3, -2, 0, -5, -3, 5, 0, 5], [-13, 7, 0, -10, -13, 10, 0, -9], [-5, 0, -3, 0, 0, 8, -2, 0], [-7, -3, 0, 12, -7, -12, 0, 5], [0, 11, 11, -5, 10, 0, 5, -5], [0, 0, -3, 0, 0, 3, 0, 0], [-1, 0, -3, 0, 0, -4, 0, 0], [19, -4, 0, 4, 19, -4, 0, 2], [7, 0, -6, 0, 0, 10, -7, 0], [0, -4, -4, 1, -3, 0, 5, -5], [0, -14, -14, -1, -12, 0, 2, -2], [-9, 0, 3, 0, 0, 0, 0, 0], [0, -6, -6, 1, -2, 0, 2, -2], [0, 8, 8, 1, -3, 0, 5, -5], [0, -8, -8, 1, -9, 0, -7, 7], [16, -8, 0, 6, 16, -6, 0, 11], [-3, 1, 0, 10, -3, -10, 0, 5], [0, 3, 3, -5, -12, 0, -5, 5], [6, 1, 0, 9, 6, -9, 0, 13], [0, 16, 16, -9, 4, 0, 8, -8], [-22, 4, 0, -7, -22, 7, 0, 0], [-4, 0, 14, 0, 0, -8, 8, 0], [-8, -9, 0, 13, -8, -13, 0, 5], [13, -4, 0, -6, 13, 6, 0, -6], [0, -9, -9, -8, 16, 0, 2, -2], [-13, 0, 11, 0, 0, -7, 5, 0], [7, 6, 0, -2, 7, 2, 0, -8], [-5, 11, 0, 1, -5, -1, 0, -2], [0, 9, 9, -9, -13, 0, 14, -14], [-26, 0, 3, 0, 0, -1, 0, 0], [3, 0, -1, 0, 0, -5, 9, 0], [0, 9, 9, -5, -15, 0, 10, -10], [-15, 7, 0, 2, -15, -2, 0, -5], [0, -6, -6, 7, -18, 0, -4, 4], [0, 6, 6, 4, -7, 0, -6, 6], [-17, -3, 0, 10, -17, -10, 0, 5], [22, -3, 0, -10, 22, 10, 0, -6], [19, -8, 0, -1, 19, 1, 0, -5], [0, 5, 5, 3, -4, 0, 9, -9], [1, 16, 0, 0, 1, 0, 0, 0], [-4, 4, 0, 12, -4, -12, 0, 1], [-10, 0, -3, 0, 0, 13, -14, 0], [0, -13, -13, 7, 11, 0, -8, 8], [25, 0, 0, 0, 0, 0, -1, 0], [0, 16, 16, -2, 3, 0, 14, -14], [0, 2, 2, 14, -18, 0, -1, 1], [-14, 5, 0, -2, -14, 2, 0, -16], [0, 5, 5, -11, -24, 0, 13, -13], [-8, 0, -16, 0, 0, 1, -9, 0], [0, -13, -13, -1, -19, 0, -4, 4], [-30, -5, 0, 3, -30, -3, 0, 3], [-2, 3, 0, 13, -2, -13, 0, -1], [27, 2, 0, 5, 27, -5, 0, 1], [16, 0, -3, 0, 0, -3, -2, 0], [0, 0, 0, -1, 10, 0, 0, 0], [9, 0, 0, 10, 9, -10, 0, -6], [27, 0, 3, 0, 0, -7, -1, 0], [0, 8, 8, -1, 0, 0, 0, 0], [-26, 0, 3, 0, 0, 4, 3, 0], [2, 12, 0, -7, 2, 7, 0, 0], [-11, -9, 0, -1, -11, 1, 0, -6], [0, 1, 1, 8, 2, 0, -10, 10], [-26, -1, 0, 7, -26, -7, 0, 7], [0, 9, 9, -6, 12, 0, 10, -10], [-1, 0, 17, 0, 0, -6, 4, 0], [-16, -2, 0, -10, -16, 10, 0, 3], [-19, 0, -7, 0, 0, -5, -7, 0], [7, 0, -3, 0, 0, 6, -2, 0], [2, 0, -12, 0, 0, 20, -12, 0], [-6, 0, -5, 0, 0, 5, 8, 0], [-16, 0, -12, 0, 0, -3, 7, 0], [0, -3, -3, 10, -2, 0, -12, 12], [0, 9, 9, 7, -2, 0, -13, 13], [27, -4, 0, 1, 27, -1, 0, -1], [-14, 0, 3, 0, 0, -19, 4, 0], [7, 0, -6, 0, 0, 11, -12, 0], [23, 0, 15, 0, 0, -5, 1, 0]]
